    workflow = null;
    canvas_manager = null;
    active_ajax_call = false;
    // jQuery onReady
    $( function() {
        
        if ( window.lt_ie_7 ) {
            show_modal(
                "Browser not supported",
                "Sorry, the workflow editor is not supported for IE6 and below."
            );
            return;
        }
        
        // Load jStore for local storage
        $.extend(jQuery.jStore.defaults, { project: 'galaxy', flash: '/static/jStore.Flash.html' })
        $.jStore.load(); // Auto-select best storage
        
        // Canvas overview management
        canvas_manager = new CanvasManager( $("#canvas-viewport"), $("#overview") );
        
        // Initialize workflow state
        reset();
        // Load the datatype info
        $.ajax( {
            url: "/workflow/get_datatypes",
            dataType: "json",
            cache: false,
            success: function( data ) {
                populate_datatype_info( data );
                // Load workflow definition
                $.ajax( {
                    url: "/workflow/load_workflow",
                    data: { id: "db2edea29d6dc127", "_": "true" },
                    dataType: 'json',
                    cache: false,
                    success: function( data ) {
                         reset();
                         workflow.from_simple( data );
                         workflow.has_changes = false;
                         workflow.fit_canvas_to_nodes();
                         scroll_to_nodes();
                         canvas_manager.draw_overview();
                         // Determine if any parameters were 'upgraded' and provide message
                         upgrade_message = ""
                         $.each( data['upgrade_messages'], function( k, v ) {
                            upgrade_message += ( "<li>Step " + ( parseInt(k) + 1 ) + ": " + workflow.nodes[k].name + "<ul>");
                            $.each( v, function( i, vv ) {
                                upgrade_message += "<li>" + vv +"</li>";
                            });
                            upgrade_message += "</ul></li>";
                         });
                         if ( upgrade_message ) {
                            show_modal( "Workflow loaded with changes",
                                        "Problems were encountered loading this workflow (possibly a result of tool upgrades). Please review the following parameters and then save.<ul>" + upgrade_message + "</ul>",
                                        { "Continue" : hide_modal } );
                         } else {
                            hide_modal();
                         }
                     },
                     beforeSubmit: function( data ) {
                         show_modal( "Loading workflow", "progress" );
                     }
                });
            }
        });
        
        // For autosave purposes
        $(document).ajaxStart( function() {
            active_ajax_call = true;
            $(document).bind( "ajaxStop.global", function() {
                active_ajax_call = false;
            });
        });
        
        $(document).ajaxError( function ( e, x ) {
            console.log( e, x );
            var message = x.responseText || x.statusText || "Could not connect to server";
            show_modal( "Server error", message, { "Ignore error" : hide_modal } );
            return false;
        });
        
        make_popupmenu( $("#workflow-options-button"), {
             "Edit Attributes" : edit_workflow_attributes,
             "Layout": layout_editor,
             "Save" : save_current_workflow,
             "Close": close_editor,
        });
        
        function layout_editor() {
            workflow.layout();
            workflow.fit_canvas_to_nodes();
            scroll_to_nodes();
            canvas_manager.draw_overview();
        };
        
        function edit_workflow_attributes() {
            workflow.clear_active_node();
            $('#edit-attributes').show();
            $('#right-content').hide();  
        };
        
        $.jStore.ready(function(engine) {
            engine.ready(function() {
                // On load, set the size to the pref stored in local storage if it exists
                overview_size = $.jStore.store("overview-size");
                if (overview_size) {
                    $("#overview-border").css( {
                        width: overview_size,
                        height: overview_size
                    });
                }
                
                // Show viewport on load unless pref says it's off
                if ($.jStore.store("overview-off")) {
                    hide_overview();
                } else {
                    show_overview();
                }
            });
        });
        
        // Stores the size of the overview into local storage when it's resized
        $("#overview-border").bind( "dragend", function( e ) {
            var op = $(this).offsetParent();
            var opo = op.offset();
            var new_size = Math.max( op.width() - ( e.offsetX - opo.left ),
                                     op.height() - ( e.offsetY - opo.top ) );
            $.jStore.store("overview-size", new_size + "px");
        });
        
        function show_overview() {
            $.jStore.remove("overview-off");
            $("#overview-border").css("right", "0px");
            $("#close-viewport").css("background-position", "0px 0px");
        }
        
        function hide_overview() {
            $.jStore.store("overview-off", true);
            $("#overview-border").css("right", "20000px");
            $("#close-viewport").css("background-position", "12px 0px");
        }
        
        // Lets the overview be toggled visible and invisible, adjusting the arrows accordingly
        $("#close-viewport").click( function() {
            $("#overview-border").css("right") == "0px" ? hide_overview() : show_overview();
        });
        
        // Unload handler
        window.onbeforeunload = function() {
            if ( workflow && workflow.has_changes ) {
                return "There are unsaved changes to your workflow which will be lost.";
            }
        };
        
        // Tool menu
        $( "div.toolSectionBody" ).hide();
        $( "div.toolSectionTitle > span" ).wrap( "<a href='#'></a>" );
        var last_expanded = null;
        $( "div.toolSectionTitle" ).each( function() { 
           var body = $(this).next( "div.toolSectionBody" );
           $(this).click( function() {
               if ( body.is( ":hidden" ) ) {
                   if ( last_expanded ) last_expanded.slideUp( "fast" );
                   last_expanded = body;
                   body.slideDown( "fast" );
               }
               else {
                   body.slideUp( "fast" );
                   last_expanded = null;
               }
           });
        });

        // Rename async.
        async_save_text("workflow-name", "workflow-name", "/workflow/rename_async?id=db2edea29d6dc127", "new_name");
        
        // Tag async. Simply have the workflow edit element generate a click on the tag element to activate tagging.
        $('#workflow-tag').click( function() {
            $('.tag-area').click();
            return false;
        });
        // Annotate async.
        async_save_text("workflow-annotation", "workflow-annotation", "/workflow/annotate_async?id=db2edea29d6dc127", "new_annotation", 25, true, 4);
    });

    // Global state for the whole workflow
    function reset() {
        if ( workflow ) {
            workflow.remove_all();
        }
        workflow = new Workflow( $("#canvas-container") );
    }
        
    function scroll_to_nodes() {
        var cv = $("#canvas-viewport");
        var cc = $("#canvas-container")
        var top, left;
        if ( cc.width() < cv.width() ) {
            left = ( cv.width() - cc.width() ) / 2;
        } else {
            left = 0;
        }
        if ( cc.height() < cv.height() ) {
            top = ( cv.height() - cc.height() ) / 2;
        } else {
            top = 0;
        }
        cc.css( { left: left, top: top } );
    }
    
    // Add a new step to the workflow by tool id
    function add_node_for_tool( id, title ) {
        var node = prebuild_node( 'tool', title, id );
        workflow.add_node( node );
        workflow.fit_canvas_to_nodes();
        canvas_manager.draw_overview();
        workflow.activate_node( node );
        $.ajax( {
            url: "/workflow/get_new_module_info", 
            data: { type: "tool", tool_id: id, "_": "true" },
            global: false,
            dataType: "json",
            success: function( data ) {
                node.init_field_data( data );
            },
            error: function( x, e ) {
                var m = "error loading field data"
                if ( x.status == 0 ) {
                    m += ", server unavailable"
                }
                node.error( m );
            }
        });
    };
    
    function add_node_for_module( type, title ) {
        node = prebuild_node( type, title );
        workflow.add_node( node );
        workflow.fit_canvas_to_nodes();
        canvas_manager.draw_overview();
        workflow.activate_node( node );
        $.ajax( {
            url: "/workflow/get_new_module_info", 
            data: { type: type, "_": "true" }, 
            dataType: "json",
            success: function( data ) {
                node.init_field_data( data );
            },
            error: function( x, e ) {
                var m = "error loading field data"
                if ( x.status == 0 ) {
                    m += ", server unavailable"
                }
                node.error( m );
            }
        });
    };

    function show_form_for_tool( text, node ) {
        $("#edit-attributes").hide();
		$("#right-content").show().html( text );   
        $("#right-content").find( "form" ).ajaxForm( {
            type: 'POST',
            dataType: 'json',
            success: function( data ) {
                node.update_field_data( data );
            },
            beforeSubmit: function( data ) {
                data.push( { name: 'tool_state', value: node.tool_state } );
                data.push( { name: '_', value: "true" } );
            }
        }).each( function() {
            form = this;
            $(this).find( "select[refresh_on_change='true']").change( function() {
                $(form).submit();
            });
            $(this).find( ".popupmenu" ).each( function() {
                var id = $(this).parents( "div.form-row" ).attr( 'id' );
                var b = $('<a class="popup-arrow" id="popup-arrow-for-' + id + '">&#9660;</a>');
                var options = {};
                $(this).find( "button" ).each( function() {
                    var name = $(this).attr( 'name' );
                    var value = $(this).attr( 'value' );
                    options[ $(this).text() ] = function() {
                        $(form).append( "<input type='hidden' name='"+name+"' value='"+value+"' />" ).submit();
                    }
                });
                b.insertAfter( this );
                $(this).remove();
                make_popupmenu( b, options );
            });
            
            // Implements auto-saving based on whether the inputs change. We consider
            // "changed" to be when a field is accessed and not necessarily modified
            // because of an issue where "onchange" is not triggered when activating
            // another node, or saving the workflow.
            $(this).find("input,textarea,select").each( function() {
                $(this).focus( function() {
                    workflow.active_form_has_changes = true;
                });
            });
        });
        
            
        // Add metadata form to tool.
        if (node) {
            var metadata_div = 
            $( "<p><div class='metadataForm'>                 <div class='metadataFormTitle'>Edit Step Attributes</div>                 <div class='form-row'>                 <label>Annotation / Notes:</label>                         <div style='margin-right: 10px;'>                         <textarea name='annotation' rows='3' style='width: 100%'>" + node.annotation + "</textarea>                             <div class='toolParamHelp'>Add an annotation or notes to this step; annotations are available when a workflow is viewed.</div>                         </div>                 </div>                 </div>");
            // See above comment: this is necessary to handle autosaving.
            var textarea = $(metadata_div).find("textarea");
            textarea.focus( function () {
                workflow.active_form_has_changes = true;
            });
            $("#right-content").find(".toolForm").after( metadata_div );
        }
    }
    
    var close_editor = function() {
        
        workflow.check_changes_in_active_form();
        if ( workflow && workflow.has_changes ) {
            do_close = function() {
                window.onbeforeunload = undefined;
                window.document.location = "/workflow"
            };
            show_modal( "Close workflow editor",
                        "There are unsaved changes to your workflow which will be lost.",
                        {
                            "Cancel" : hide_modal,
                            "Save Changes" : function() {
                                save_current_workflow( null, do_close );
                            }
                        }, {
                            "Don't Save": do_close
                        } );
        } else {
            window.document.location = "/workflow"
        }
    }
    
    var save_current_workflow = function ( eventObj, success_callback ) {
        show_modal( "Saving workflow", "progress" );
        workflow.check_changes_in_active_form();
        if (!workflow.has_changes) {
            hide_modal();
            if ( success_callback ) {
                success_callback();
            }
            return;
        }
        var savefn = function(callback) {
            $.ajax( {
                url: "/workflow/save_workflow",
                type: "POST",
                data: {
                    id: "db2edea29d6dc127",
                    workflow_data: function() { return JSON.stringify( workflow.to_simple() ) },
                    "_": "true"
                },
                dataType: 'json',
                success: function( data ) { 
                    var body = $("<div></div>").text( data.message );
                    if ( data.errors ) {
                        body.addClass( "warningmark" )
                        var errlist = $( "<ul/>" );
                        $.each( data.errors, function( i, v ) {
                            $("<li></li>").text( v ).appendTo( errlist );
                        });
                        body.append( errlist );
                    } else {
                        body.addClass( "donemark" );
                    }
                    workflow.name = data.name;
                    workflow.has_changes = false;
                    workflow.stored = true;
                    if ( data.errors ) {
                        show_modal( "Saving workflow", body, { "Ok" : hide_modal } );
                    } else {
                        if (callback) {
                            callback();
                        }
                        hide_modal();
                    }
                }
            });
        }
        
        // We bind to ajaxStop because of auto-saving, since the form submission ajax
        // call needs to be completed so that the new data is saved
        if (active_ajax_call) {
            $(document).bind('ajaxStop.save_workflow', function() {
                $(document).unbind('ajaxStop.save_workflow');
                savefn();
                $(document).unbind('ajaxStop.save_workflow'); // IE7 needs it here
                active_ajax_call = false;
            });
        } else {
            savefn(success_callback);
        }
    }

        
    